@@ -25,7 +25,7 @@ import (
 
 //export compilerCallback
 func compilerCallback(errorLevel C.int, filename *C.char, linenumber C.int, rule *C.YR_RULE, message *C.char, userData unsafe.Pointer) {
-	c := cgoHandle(userData).Value().(*Compiler)
+	c := cgoHandle(*(*uintptr)(userData)).Value().(*Compiler)
 	msg := CompilerMessage{
 		Filename: C.GoString(filename),
 		Line:     int(linenumber),
@@ -54,7 +54,7 @@ type Compiler struct {
 	Errors   []CompilerMessage
 	Warnings []CompilerMessage
 	// used for include callback
-	callbackData cgoHandle
+	callbackData *cgoHandle
 	cptr         *C.YR_COMPILER
 }
 
@@ -73,7 +73,8 @@ func NewCompiler() (*Compiler, error) {
 	if err := newError(C.yr_compiler_create(&yrCompiler)); err != nil {
 		return nil, err
 	}
-	c := &Compiler{cptr: yrCompiler}
+	c := &Compiler{cptr: yrCompiler, callbackData: (*cgoHandle)(C.malloc(C.size_t(unsafe.Sizeof(cgoHandle(0)))))}
+	*c.callbackData = 0
 	runtime.SetFinalizer(c, (*Compiler).Destroy)
 	return c, nil
 }
@@ -86,14 +87,21 @@ func (c *Compiler) Destroy() {
 		C.yr_compiler_destroy(c.cptr)
 		c.cptr = nil
 	}
+	if c.callbackData != nil {
+		C.free(unsafe.Pointer(c.callbackData))
+		c.callbackData = nil
+	}
 	runtime.SetFinalizer(c, nil)
 }
 
-func (c *Compiler) setCallbackData(ptr cgoHandle) {
-	if c.callbackData != 0 {
+func (c *Compiler) setCallbackData(cb CompilerIncludeFunc) {
+	if *c.callbackData != 0 {
 		c.callbackData.Delete()
+		*c.callbackData = 0
+	}
+	if cb != nil {
+		*c.callbackData = cgoNewHandle(cb)
 	}
-	c.callbackData = ptr
 }
 
 // AddFile compiles rules from a file. Rules are added to the
@@ -114,7 +122,8 @@ func (c *Compiler) AddFile(file *os.File, namespace string) (err error) {
 	defer C.free(unsafe.Pointer(filename))
 	id := cgoNewHandle(c)
 	defer id.Delete()
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), unsafe.Pointer(id))
+	cbp := unsafe.Pointer(&id)
+	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), cbp)
 	numErrors := int(C._yr_compiler_add_fd(c.cptr, C.int(file.Fd()), ns, filename))
 	if numErrors > 0 {
 		var buf [1024]C.char
@@ -123,6 +132,7 @@ func (c *Compiler) AddFile(file *os.File, namespace string) (err error) {
 		err = errors.New(msg)
 	}
 	runtime.KeepAlive(c)
+	runtime.KeepAlive(cbp)
 	return
 }
 
@@ -144,7 +154,8 @@ func (c *Compiler) AddString(rules string, namespace string) (err error) {
 	defer C.free(unsafe.Pointer(crules))
 	id := cgoNewHandle(c)
 	defer id.Delete()
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), unsafe.Pointer(id))
+	cbp := unsafe.Pointer(&id)
+	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), cbp)
 	numErrors := int(C.yr_compiler_add_string(c.cptr, crules, ns))
 	if numErrors > 0 {
 		var buf [1024]C.char
@@ -153,6 +164,7 @@ func (c *Compiler) AddString(rules string, namespace string) (err error) {
 		err = errors.New(msg)
 	}
 	runtime.KeepAlive(c)
+	runtime.KeepAlive(cbp)
 	return
 }
 
@@ -205,7 +217,7 @@ func (c *Compiler) GetRules() (*Rules, error) {
 
 //export includeCallback
 func includeCallback(name, filename, namespace *C.char, userData unsafe.Pointer) *C.char {
-	callbackFunc := cgoHandle(userData).Value().(CompilerIncludeFunc)
+	callbackFunc := cgoHandle(*(*uintptr)(userData)).Value().(CompilerIncludeFunc)
 	if buf := callbackFunc(
 		C.GoString(name), C.GoString(filename), C.GoString(namespace),
 	); buf != nil {
@@ -249,13 +261,12 @@ func (c *Compiler) SetIncludeCallback(cb CompilerIncludeFunc) {
 		c.DisableIncludes()
 		return
 	}
-	id := cgoNewHandle(cb)
-	c.setCallbackData(id)
+	c.setCallbackData(cb)
 	C.yr_compiler_set_include_callback(
 		c.cptr,
 		C.YR_COMPILER_INCLUDE_CALLBACK_FUNC(C.includeCallback),
 		C.YR_COMPILER_INCLUDE_FREE_FUNC(C.freeCallback),
-		unsafe.Pointer(id),
+		unsafe.Pointer(c.callbackData),
 	)
 	runtime.KeepAlive(c)
 	return
@@ -265,7 +276,7 @@ func (c *Compiler) SetIncludeCallback(cb CompilerIncludeFunc) {
 // See yr_compiler_set_include_callbacks.
 func (c *Compiler) DisableIncludes() {
 	C.yr_compiler_set_include_callback(c.cptr, nil, nil, nil)
-	c.setCallbackData(0)
+	c.setCallbackData(nil)
 	runtime.KeepAlive(c)
 	return
 }
